@@ -44,6 +44,11 @@ public:
         raise();
     }
 
+    int tabIndex() const
+    {
+        return m_tabIdx;
+    }
+
 private:
     void doResize()
     {
@@ -407,8 +412,8 @@ KPageViewPrivate::KPageViewPrivate(KPageView *_parent)
     , layout(nullptr)
     , stack(nullptr)
     , titleWidget(nullptr)
-    , searchLineEditContainer(new QWidget())
-    , searchLineEdit(new QLineEdit())
+    , searchLineEditContainer(nullptr)
+    , searchLineEdit(nullptr)
     , view(nullptr)
 {
 }
@@ -440,6 +445,7 @@ void KPageViewPrivate::init()
     layout->setColumnStretch(1, 1);
     layout->setRowStretch(3, 1);
 
+    searchLineEdit = new QLineEdit(defaultWidget);
     searchTimer.setInterval(400);
     searchTimer.setSingleShot(true);
     searchTimer.callOnTimeout(q, [this] {
@@ -448,11 +454,12 @@ void KPageViewPrivate::init()
     q->setFocusProxy(searchLineEdit);
     searchLineEdit->setPlaceholderText(KPageView::tr("Search…", "@info:placeholder"));
     searchLineEdit->setClearButtonEnabled(true);
-    searchLineEdit->setParent(defaultWidget);
     auto a = new QAction(q);
     a->setIcon(QIcon::fromTheme(QStringLiteral("search")));
     searchLineEdit->addAction(a, QLineEdit::LeadingPosition);
     q->connect(searchLineEdit, &QLineEdit::textChanged, &searchTimer, QOverload<>::of(&QTimer::start));
+
+    searchLineEditContainer = new QWidget(q);
     auto containerLayout = new QVBoxLayout(searchLineEditContainer);
     containerLayout->setContentsMargins({});
     containerLayout->setSpacing(0);
@@ -600,22 +607,18 @@ void KPageViewPrivate::onSearchTextChanged()
         // @p w lives
         auto parent = w->parentWidget();
         TabWidgetAndPage p = {nullptr, nullptr};
-        if (auto tw = qobject_cast<QTabWidget *>(parent)) {
-            p.first = tw;
-        }
         QVarLengthArray<QWidget *, 8> parentChain;
+        parentChain << parent;
         while (parent) {
-            if (!p.first) {
-                if (auto tw = qobject_cast<QTabWidget *>(parent)) {
-                    if (parentChain.size() >= 3) {
-                        // last == QTabWidget
-                        // second last == QStackedWidget of QTabWidget
-                        // third last => the widget we want
-                        p.second = parentChain.value((parentChain.size() - 1) - 2);
-                    }
-                    p.first = tw;
-                    break;
+            if (auto tw = qobject_cast<QTabWidget *>(parent)) {
+                if (parentChain.size() >= 3) {
+                    // last == QTabWidget
+                    // second last == QStackedWidget of QTabWidget
+                    // third last => the widget we want
+                    p.second = parentChain.value((parentChain.size() - 1) - 2);
                 }
+                p.first = tw;
+                break;
             }
             parent = parent->parentWidget();
             parentChain << parent;
@@ -636,7 +639,14 @@ void KPageViewPrivate::onSearchTextChanged()
                     // qDebug() << page << tabWidget << "not found" << w;
                     continue;
                 }
-                m_searchMatchOverlays << new SearchMatchOverlay(tabWidget->tabBar(), idx);
+
+                const bool alreadyOverlayed =
+                    std::any_of(m_searchMatchOverlays.cbegin(), m_searchMatchOverlays.cend(), [tabbar = tabWidget->tabBar(), idx](SearchMatchOverlay *overlay) {
+                        return idx == overlay->tabIndex() && tabbar == overlay->parentWidget();
+                    });
+                if (!alreadyOverlayed) {
+                    m_searchMatchOverlays << new SearchMatchOverlay(tabWidget->tabBar(), idx);
+                }
             }
         }
     }
